@@ -1,4 +1,5 @@
 import csv
+import numpy as np
 from annogesiclib.gff3 import Gff3Parser
 from annogesiclib.coverage_detection import coverage_comparison, check_tex, get_repmatch
 from annogesiclib.lib_reader import read_libs, read_wig
@@ -15,6 +16,8 @@ def import_data(row):
 
 
 def compare_ta(terms, tas, fuzzy):
+    '''Compare transcript and terminator to 
+    find the expressed terminator'''
     for term in terms:
         for ta in tas:
             start = ta.start - fuzzy
@@ -35,6 +38,7 @@ def compare_ta(terms, tas, fuzzy):
 
 
 def compare_transtermhp(hps, fr_terms):
+    '''compare two methods - gene converged and TransTermHP'''
     terms = []
     for term in fr_terms:
         detect = False
@@ -83,6 +87,7 @@ def compare_transtermhp(hps, fr_terms):
     return terms
 
 def compare_replicates(term_covers, template_texs, cond, args_term):
+    '''check the cutoff of replicate match'''
     detect_num = 0
     term_datas = []
     diff_cover = -1
@@ -109,20 +114,26 @@ def compare_replicates(term_covers, template_texs, cond, args_term):
     return diff_cover, diff, term_datas, detect_num
 
 
-def coverage2term(covers, term, hl_covers, hl_poss, strand,
-                  term_covers, track, args_term):
+def coverage2term(covers, term, hl_covers, hl_poss, strand, term_covers,
+                  track, args_term, start_plus, end_minus, lib_type):
+    '''It is for get the highest and lowest coverage'''
     first = True
+    pos = 0
     for cover in covers:
-        if (term["start"] <= cover["pos"] + args_term.fuzzy) and (
-                term["end"] >= cover["pos"] - args_term.fuzzy):
+        if strand == "+":
+            cover_pos = start_plus + pos
+        else:
+            cover_pos = end_minus - pos
+        if (term["start"] <= cover_pos + args_term.fuzzy) and (
+                term["end"] >= cover_pos - args_term.fuzzy):
             first = coverage_comparison(cover, hl_covers, hl_poss,
-                                        first, strand)
+                                        first, strand, cover_pos)
         else:
             if (strand == "+") and (
-                    cover["pos"] > term["end"] + args_term.fuzzy):
+                    cover_pos > term["end"] + args_term.fuzzy):
                 break
             elif (strand == "-") and (
-                    cover["pos"] < term["start"] - args_term.fuzzy):
+                    cover_pos < term["start"] - args_term.fuzzy):
                 break
         if (first is not True) and (hl_covers["high"] > 0):
             if ((hl_covers["low"] / hl_covers["high"]) <
@@ -132,11 +143,24 @@ def coverage2term(covers, term, hl_covers, hl_poss, strand,
                     "track": track, "high": hl_covers["high"],
                     "low": hl_covers["low"], "detect": "True",
                     "diff": (hl_covers["high"] - hl_covers["low"]),
-                    "type": cover["type"]})
+                    "type": lib_type})
                 break
+        pos += 1
 
 
+def check_start_and_end(term, args_term, covers):
+    if (term["start"] - args_term.fuzzy - 2) < 0:
+        start = 0
+    else:
+        start = term["start"] - args_term.fuzzy - 2
+    if (term["end"] + args_term.fuzzy + 1) > len(covers):
+        end = len(covers)
+    else:
+        end = term["end"] + args_term.fuzzy + 1
+    return start, end
+
 def get_coverage(term, wigs, strand, template_texs, args_term):
+    '''get proper coverage to check the coverage decrease'''
     hl_poss = {"high": 0, "low": 0}
     hl_covers = {"high": 0, "low": 0}
     term_datas = {}
@@ -149,15 +173,17 @@ def get_coverage(term, wigs, strand, template_texs, args_term):
                 term_covers = []
                 term_datas[cond] = []
                 detect_nums[cond] = 0
-                for track, covers in tracks.items():
+                for lib_name, covers in tracks.items():
+                    track = lib_name.split("|")[-3]
+                    lib_strand = lib_name.split("|")[-2]
+                    lib_type = lib_name.split("|")[-1]
+                    c_start, c_end = check_start_and_end(term, args_term, covers)
+                    covers = covers[c_start: c_end]
                     if strand == "-":
-                        covers = reversed(covers[term["start"] -
-                                          args_term.fuzzy - 2:
-                                          term["end"] + args_term.fuzzy + 1])
-                    elif strand == "+":
-                        covers = covers[term["start"] - 2: term["end"] + 1]
+                        covers = covers[::-1]
                     coverage2term(covers, term, hl_covers, hl_poss,
-                                  strand, term_covers, track, args_term)
+                                  strand, term_covers, track, args_term,
+                                  c_start, c_end, lib_type)
                 if len(term_covers) != 0:
                     tmp_cov, tmp_diff, term_datas[cond], detect_nums[cond] = (
                             compare_replicates(term_covers, template_texs,
@@ -185,6 +211,8 @@ def get_coverage(term, wigs, strand, template_texs, args_term):
 
 
 def compare_term(term, terms):
+    '''For the terminators which are associated with the gene, it 
+    will try to find the best one based on secondary structure'''
     if len(terms) != 0:
         for tmp in terms:
             if term["miss"] < tmp["miss"]:
@@ -212,6 +240,9 @@ def compare_term(term, terms):
 
 
 def first_term(strand, term, detect_terms, detect):
+    '''for the first and latest terminators, 
+    we only need to check parent gene for one side of terminator
+    and also need to take care of the terminal end'''
     if (strand == "+"):
         if (term["detect_p"]):
             detect_terms["detect"].append(term)
@@ -239,6 +270,7 @@ def get_attribute_string(num, name, parent, diff, term, coverage, method):
 
 
 def print_table(term, out_t, args_term):
+    '''Print to table'''
     first = True
     if (term["express"] == "True") and \
        (term["diff_cover"] != -1):
@@ -275,6 +307,7 @@ def print_table(term, out_t, args_term):
 
 
 def print2file(num, term, coverage, parent, out, out_t, method, args_term):
+    '''Print to gff file and table'''
     name = 'terminator_%0*d' % (5, num)
     if ("detect_num" in term.keys()) and \
        (term["diff_cover"] != -1):
@@ -311,6 +344,7 @@ def print2file(num, term, coverage, parent, out, out_t, method, args_term):
 
 
 def print_detect_undetect(terms, num, out, out_t, detect, args_term):
+    '''For controlling the output of different strand of terminator'''
     for term in terms:
         if term["strand"] == "+":
             print2file(num, term, detect, term["parent_p"], out,
@@ -325,6 +359,7 @@ def print_detect_undetect(terms, num, out, out_t, detect, args_term):
 
 def term_validation(pre_term, term, detect, detect_terms, out,
                     out_t, num, args_term):
+    '''Classification of terminators'''
     if (pre_term["name"] != term["name"]) or (args_term.keep_multi):
         if detect:
             num = print_detect_undetect(detect_terms["detect"], num, out,
@@ -381,6 +416,7 @@ def print_term(terms, out, out_t, args_term):
 
 
 def del_repeat_term(terms):
+    '''delete the repeat terminators'''
     first = True
     new_terms = []
     for term in terms:
@@ -457,6 +493,7 @@ def read_data(gff_file, tran_file, tranterm_file, seq_file, term_table):
 
 
 def compute_wig(wig_file, libs, terms, strand, texs, args_term):
+    '''Get the coverage information to specific terminator'''
     wigs = {}
     wigs = read_wig(wig_file, strand, libs)
     for term in terms:
@@ -472,6 +509,7 @@ def compute_wig(wig_file, libs, terms, strand, texs, args_term):
 def detect_coverage(term_table, gff_file, tran_file, seq_file,
                     wig_f_file, wig_r_file, tranterm_file, wig_folder,
                     output_file, output_table, args_term):
+    '''For detecting the coverage of terminator'''
     gffs, tas, hps, fr_terms, seq = read_data(gff_file, tran_file,
                                               tranterm_file, seq_file,
                                               term_table)
