@@ -14,6 +14,7 @@ from annogesiclib.compare_tran_term import compare_term_tran
 
 
 class Terminator(object):
+    '''detection of terminator'''
 
     def __init__(self, args_term):
         self.multiparser = Multiparser()
@@ -200,6 +201,7 @@ class Terminator(object):
         return merge_wigs
 
     def _merge_sRNA(self, sRNAs, prefixs, gff_path):
+        '''searching the terminator with sRNA information'''
         if sRNAs is not None:
             self.multiparser.parser_gff(sRNAs, "sRNA")
             self.helper.check_make_folder(self.tmps["merge"])
@@ -273,6 +275,7 @@ class Terminator(object):
 
     def _compute_intersection_forward_reverse(
             self, prefixs, merge_path, wig_path, merge_wigs, args_term):
+        '''the approach for searching gene converged region terminator'''
         for prefix in prefixs:
             tmp_seq = os.path.join(args_term.out_folder,
                                    "_".join(["inter_seq", prefix]))
@@ -386,21 +389,29 @@ class Terminator(object):
             if file_.endswith(".gff"):
                 self.helper.check_uni_attributes(os.path.join(folder, file_))
 
-    def _compare_term_tran(self, args_term):
+    def _compare_term_tran(self, args_term, prefixs):
+        '''searching the associated terminator to transcript'''
         self.multiparser.combine_gff(args_term.gffs, self.tran_path,
                                      None, "transcript")
+        prefixs = []
+        for file_ in os.listdir(self.tran_path):
+            if file_.endswith("_transcript.gff"):
+                prefixs.append(file_.replace("_transcript.gff", ""))
         for type_ in ("best", "express", "all_candidates"):
             compare_term_tran(self.tran_path,
                               os.path.join(self.outfolder["term"], type_),
                               args_term.fuzzy_up_ta, args_term.fuzzy_down_ta,
-                              args_term.out_folder, "terminator")
-            shutil.move(
-                os.path.join(
-                    args_term.out_folder, "statistics",
-                    "stat_comparison_terminator_transcript.csv"),
-                os.path.join(
-                    args_term.out_folder, "statistics",
-                    "stat_comparison_terminator_transcript_" + type_ + ".csv"))
+                              args_term.out_folder, "terminator",
+                              self.outfolder["term"], args_term.trans)
+            for prefix in prefixs:
+                shutil.move(
+                    os.path.join(
+                        args_term.out_folder, "statistics",
+                        "stat_compare_transcript_terminator_" + prefix + ".csv"),
+                    os.path.join(
+                        args_term.out_folder, "statistics",
+                        "_".join(["stat_compare_terminator_transcript", prefix,
+                                  type_ + ".csv"])))
 
     def run_terminator(self, args_term):
         self._check_gff_file(args_term.gffs)
@@ -424,5 +435,5 @@ class Terminator(object):
                 prefixs, merge_path, args_term.wig_path,
                 args_term.merge_wigs, args_term)
         self._compute_stat(args_term)
-        self._compare_term_tran(args_term)
+        self._compare_term_tran(args_term, prefixs)
         self._remove_tmp_file(args_term.merge_wigs, args_term)
